/****************************************************************/
/*                                                              */
/*            Advanced Navigation Kinematica API v1.2           */
/*              Copyright 2018, Advanced Navigation             */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2018 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.kinematica.api;

import java.io.File;
import java.util.ArrayList;

import com.advancednavigation.kinematica.api.data.Configuration;
import com.advancednavigation.kinematica.api.data.DataSet;
import com.advancednavigation.kinematica.api.json.DataSetJSON;
import com.advancednavigation.kinematica.api.json.KinematicaFileJSON;
import com.advancednavigation.kinematica.api.json.LogJSON;
import com.advancednavigation.kinematica.api.json.StatusJSON;

public interface KinematicaApiInterface
{
	/**
	 * Exception for the KinematicaApi
	 */
	public static class KinematicaException extends Exception
	{
		private static final long serialVersionUID = 1L;

		public KinematicaException(String s)
		{
			super(s);
		}
	}

	/**
	 * Sets the API key to be used in future requests.
	 * 
	 * @param apiKey
	 */
	public void setApiKey(String apiKey);

	/**
	 * Gets a list of data sets on the account
	 * 
	 * @return An ArrayList of DataSet
	 * @throws Exception
	 */
	public ArrayList<DataSetJSON> allDataSets() throws Exception;

	/**
	 * Creates a new data set
	 * 
	 * @param dataSet
	 *            : The data set to be created
	 * @return The data set created. (The data set's id is required for future api calls).
	 * @throws Exception
	 */
	public DataSetJSON newDataSet(DataSet dataSet) throws Exception;

	/**
	 * Retrieves list of antenna options
	 * 
	 * @return A list of base station antenna options
	 * @throws Exception
	 */
	public ArrayList<String> antennaOptions() throws Exception;

	/**
	 * Edits the data set configuration using the configuration provided
	 * 
	 * @param id
	 *            : The data set id
	 * @param configuration
	 *            : The data set configuration
	 * @return Returns true on success, false on failure
	 * @throws Exception
	 */
	public boolean editConfiguration(int id, Configuration configuration) throws Exception;

	/**
	 * Processes the data set
	 * 
	 * @param id
	 *            : The data set id
	 * @return Returns true on success, false on failure
	 * @throws Exception
	 */
	public boolean processDataSet(int id) throws Exception;

	/**
	 * Gets the data set status
	 * 
	 * @param id
	 *            : The data set id
	 * @return The status of the data set.
	 * @throws Exception
	 */
	public StatusJSON dataSetStatus(int id) throws Exception;

	/**
	 * Gets a list of logs for the data set after "time". 
	 * Logs are limited to 100 per query. 
	 * Multiple calls are required to retrieve the entire data set's log. 
	 * The first call should provided a time of 0.
	 * Future calls should use the time from the last log.
	 * 
	 * @param id
	 *            : The data set id
	 * @param time
	 *            : The last log time.
	 * @return
	 * @throws Exception
	 */
	public ArrayList<LogJSON> dataSetLogsAfterTime(int id, long time) throws Exception;

	/**
	 * Gets a list of the files for that data set. 
	 * This includes any user uploaded files, rinex files used and the post-process result files
	 * 
	 * @param id
	 *            : The data set id.
	 * @return A list of the data set's files.
	 * @throws Exception
	 */
	public ArrayList<KinematicaFileJSON> dataSetFiles(int id) throws Exception;

	/**
	 * Downloads a file from a dataset into the specified directory.
	 * 
	 * @param id
	 *            : The data set id.
	 * @param filename
	 *            : The file name.
	 * @param directory
	 *            : The directory to store the file.
	 * @return The file downloaded.
	 * @throws Exception
	 */
	public File downloadDataSetFile(int id, String filename, File directory) throws Exception;

}
